"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertDocumentToPdf = exports.renderDocument = exports.buildOptions = exports.isWordDocument = exports.withTempDir = exports.withTempFile = void 0;
const promises_1 = __importDefault(require("fs/promises"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const carbone_1 = __importDefault(require("carbone"));
const withTempFile = (fn) => withTempDir((dir) => fn(path_1.default.join(dir, 'file')));
exports.withTempFile = withTempFile;
const withTempDir = async (fn) => {
    const dir = await promises_1.default.mkdtemp((await promises_1.default.realpath(os_1.default.tmpdir())) + path_1.default.sep);
    try {
        return await fn(dir);
    }
    finally {
        await promises_1.default.rm(dir, { recursive: true });
    }
};
exports.withTempDir = withTempDir;
const isWordDocument = (data) => data.mimeType === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document';
exports.isWordDocument = isWordDocument;
const buildOptions = (node, index) => {
    const additionalFields = node.getNodeParameter('options', index);
    let options = {};
    if (additionalFields.timezone)
        options.timezone = additionalFields.timezone;
    if (additionalFields.lang)
        options.lang = additionalFields.lang;
    if (additionalFields.variableStr)
        options.variableStr = additionalFields.variableStr;
    if (additionalFields.complement)
        options.complement = JSON.parse(additionalFields.complement);
    if (additionalFields.enum)
        options.enum = JSON.parse(additionalFields.enum);
    if (additionalFields.translations)
        options.translations = JSON.parse(additionalFields.translations);
    return options;
};
exports.buildOptions = buildOptions;
const renderDocument = async (document, context, options) => {
    return withTempFile(async (file) => {
        await promises_1.default.writeFile(file, document);
        return await new Promise((resolve, reject) => {
            carbone_1.default.render(file, context, options, function (err, result) {
                if (err) {
                    reject(err);
                }
                if (typeof result === 'string') {
                    resolve(Buffer.from(result, 'utf-8'));
                }
                else {
                    resolve(result);
                }
            });
        });
    });
};
exports.renderDocument = renderDocument;
const convertDocumentToPdf = async (document) => {
    var options = {
        convertTo: 'pdf',
        extension: 'docx',
    };
    return await new Promise((resolve, reject) => {
        carbone_1.default.convert(document, options, function (err, result) {
            if (err) {
                reject(err);
            }
            resolve(result);
        });
    });
};
exports.convertDocumentToPdf = convertDocumentToPdf;
//# sourceMappingURL=CarboneUtils.js.map